#
# Django Gems.
#
# Copyright (C) 2012 Pier Luigi Fiorini
#
# Combined from following two snippets:
# - crucialfelix (_unique_slugify) http://www.djangosnippets.org/snippets/1321/
# - GaretJax (AutoSlugField) http://www.djangosnippets.org/snippets/728/ 
#
# Improved from those by Ciantic, 2010.
# Maintained by Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import re
from django.db.models import fields
from django.template.defaultfilters import slugify

def _unique_slugify(instance, value, slug_field_name='slug', queryset=None,
					slug_separator='-'):
	"""
	Calculates a unique slug of ``value`` for an instance.

	:param slug_field_name: Should be a string matching the name of the field to
		store the slug in (and the field to check against for uniqueness).

	:param queryset: usually doesn't need to be explicitly provided - it'll
		default to using the ``.all()`` queryset from the model's default
		manager.
	
	"""
	slug_field = instance._meta.get_field(slug_field_name)
	
	slug_len = slug_field.max_length

	# Sort out the initial slug. Chop its length down if we need to.
	slug = slugify(value)
	if slug_len:
		slug = slug[:slug_len]
	slug = _slug_strip(slug, slug_separator)
	original_slug = slug

	# Create a queryset, excluding the current instance.
	if queryset is None:
		queryset = instance.__class__._default_manager.all()
		if instance.pk:
			queryset = queryset.exclude(pk=instance.pk)

	# Find a unique slug. If one matches, at '-2' to the end and try again
	# (then '-3', etc).
	next = 2
	while not slug or queryset.filter(**{slug_field_name: slug}):
		slug = original_slug
		end = '-%s' % next
		if slug_len and len(slug) + len(end) > slug_len:
			slug = slug[:slug_len-len(end)]
			slug = _slug_strip(slug, slug_separator)
		slug = '%s%s' % (slug, end)
		next += 1

	setattr(instance, slug_field.attname, slug)
	return slug

def _slug_strip(value, separator=None):
	"""
	Cleans up a slug by removing slug separator characters that occur at the
	beginning or end of a slug.

	If an alternate separator is used, it will also replace any instances of the
	default '-' separator with the new separator.
	
	"""
	if separator == '-' or not separator:
		re_sep = '-'
	else:
		re_sep = '(?:-|%s)' % re.escape(separator)
		value = re.sub('%s+' % re_sep, separator, value)
	return re.sub(r'^%s+|%s+$' % (re_sep, re_sep), '', value)

class AutoSlugField(fields.SlugField):
	"""Auto slug field, creates unique slug for model."""
	
	def __init__(self, prepopulate_from, *args, **kwargs):
		"""Create auto slug field.
		
		If field is unique, the uniqueness of the slug is ensured from existing
		slugs by adding extra number at the end of slug.
		
		If field has slug given, it is used instead. If you want to re-generate
		the slug, just set it :const:`None` or :const:`""` so it will be re-
		generated automatically.
		
		:param prepopulate_from: Must be assigned to list of field names which
			are used to prepopulate automatically. 
		
		:type prepopulate_from: sequence
		"""
		self.prepopulate_separator = kwargs.get("prepopulate_separator", u"-")
		self.prepopulate_from = prepopulate_from
		kwargs["blank"] = True
		super(fields.SlugField, self).__init__(*args, **kwargs)
	
	def pre_save(self, model_instance, add): #@UnusedVariable
		"""Pre-save event"""
		current_slug = getattr(model_instance, self.attname)
		
		# Use current slug instead, if it is given.
		# Assumption: There are no empty slugs.
		if not (current_slug is None or current_slug == ""):
			slug = current_slug
		else:
			slug = self.prepopulate_separator.\
						join(unicode(getattr(model_instance, prepop)) 
							 for prepop in self.prepopulate_from)
		
		if self.unique:
			return _unique_slugify(model_instance, value=slug, 
								   slug_field_name=self.attname)
		else:
			return slugify(slug)[:self.max_length]



#
# Django Gems.
#
# Copyright (C) 2012 Pier Luigi Fiorini
# Copyright (C) 2008 rizumu
#
# Author(s):
#   Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#   rizumu
#
# Taken from:
#	http://djangosnippets.org/snippets/1118/
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from django import template
from django.template import Context, loader
from django.conf import settings

register = template.Library()

@register.simple_tag
def analytics():
	"""
	This template tag allows easy inclusion of Google Analytics script.
	Define ANALYTICS_ID in your settings.py and put the script into templates/analytics/analytics.html,
	an example follows:

	<script type="text/javascript">
		var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
		document.write(unescape("%3Cscript src='" + gaJsHost + "google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E"));
	</script>
	<script type="text/javascript">
		var pageTracker = _gat._getTracker("{{ analytics_code }}");
		pageTracker._trackPageview();
	</script>
	"""
	analytics_id = getattr(settings, "ANALYTICS_ID", None)
	if analytics_id and analytics_id.strip() != "":
		t = loader.get_template ("analytics/analytics.html")
		c = Context({
			"analytics_code": analytics_id
		})
		return t.render(c)
	return ""



#
# Django Gems.
#
# Copyright (C) 2012 Pier Luigi Fiorini
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

try:
	from cStringIO import StringIO
except ImportError:
	from StringIO import StringIO
from django.http import HttpResponse

def xls_convert(array):
	"""
	This function creates a workbook from an array of arrays.
	Each item is a row represented as another array, for example:
	array = [
		["A1", "B1", "C1"],
		["A2", "B2", "C2"]
	]
	"""
	import xlwt

	wb = xlwt.Workbook()
	ws = wb.add_sheet("Foglio")

	row_count = 0
	for row in array:
		for col in range(len(row)):
			ws.write(row_count, col, row[col])
		row_count += 1

	buffer = StringIO()
	wb.save(buffer)
	data = buffer.getvalue()
	buffer.close()
	return data

def xls_view(func):
	"""
	Decorator that convers data into Excel.
	"""
	def _wrap(request, *args, **kwargs):
		try:
			(filename, array) = func(request, *args, **kwargs)
		except KeyboardInterrupt:
			# Allow keyboard interrupts through for debugging.
			raise
		response = HttpResponse(mimetype="application/vnd.ms-excel")
		response["Content-Disposition"] = "attachment; filename=" + filename
#		workbook = xls_convert(array)
#		workbook.save(response)
		data = xls_convert(array)
		response.write(data)
		return response
	return _wrap



#
# Django Gems.
#
# Copyright (C) 2011 Pier Luigi Fiorini
# Copyright (C) 2009 EveryBlock
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Adapted from
#	http://www.holovaty.com/writing/django-two-phased-rendering/
#

from django import template

register = template.Library()

def raw(parser, token):
	# Whatever is between {% raw %} and {% endraw %} will be preserved as
	# raw, unrendered template code.
	text = []
	parse_until = "endraw"
	tag_mapping = {
		template.TOKEN_TEXT: ("", ""),
		template.TOKEN_VAR: ("{{", "}}"),
		template.TOKEN_BLOCK: ("{%", "%}"),
		template.TOKEN_COMMENT: ("{#", "#}"),
	}
	# By the time this template tag is called, the template system has already
	# lexed the template into tokens. Here, we loop over the tokens until
	# {% endraw %} and parse them to TextNodes. We have to add the start and
	# end bits (e.g. "{{" for variables) because those have already been
	# stripped off in a previous part of the template-parsing process.
	while parser.tokens:
		token = parser.next_token()
		if token.token_type == template.TOKEN_BLOCK and token.contents == parse_until:
			return template.TextNode(u"".join(text))
		start, end = tag_mapping[token.token_type]
		text.append(u"%s%s%s" % (start, token.contents, end))
	parser.unclosed_block_tag(parse_until)
raw = register.tag(raw)



#
# Django Gems.
#
# Copyright (C) 2011 Pier Luigi Fiorini
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import logging

class LogFileHandler(logging.StreamHandler):
	"""
	Log handler that writes on files.
	"""
	def __init__(self, logfile=None):
		stream = open(logfile, "w+")
		logging.StreamHandler.__init__(self, stream)



#
# Django Gems.
#
# Copyright (C) 2011 Pier Luigi Fiorini
# Copyright (C) 2009 Gerry Eisenhaur
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

#
# Here's a brief list of changes made by Pier Luigi Fiorini.
#
# Options added:
#   - distinct
#   - start_empty
#
# Methods added:
#
# Other changes:
#   - field_to_colmodel(): editable set to False
#   - get_queryset(): queryset filtered by all fields in get_field_names()
#   - filter_items(): use self.get_model() so that self.model is set according
#                     to the queryset when not explicitely assigned by user
#   - filter_items(): use lookup_foreign_key_field() to allow searches on
#                     foreign keys.
#

import operator
from django.db import models
from django.core.exceptions import FieldError, ImproperlyConfigured
from django.core.paginator import Paginator, InvalidPage
from django.utils import simplejson as json
from django.utils.encoding import smart_str
from django.http import Http404
from json import json_encode

import logging
logger = logging.getLogger(__name__)

class JqGrid(object):
	queryset = None
	model = None
	fields = []
	allow_empty = True
	distinct = True
	start_empty = False

	pager_id = "#pager"
	url = None
	caption = None
	colmodel_overrides = {}

	def get_queryset(self, request):
		if hasattr(self, "queryset") and self.queryset is not None:
			queryset = self.queryset.values(*self.get_field_names())._clone()
		elif hasattr(self, "model") and self.model is not None:
			queryset = self.model.objects.values(*self.get_field_names())
		else:
			raise ImproperlyConfigured("No queryset or model defined.")
		if self.distinct:
			self.queryset = queryset.distinct()
		else:
			self.queryset = queryset
		return self.queryset

	def get_full_queryset(self, request):
		if hasattr(self, "queryset") and self.queryset is not None:
			queryset = self.queryset._clone()
		elif hasattr(self, "model") and self.model is not None:
			queryset = self.model.objects
		else:
			raise ImproperlyConfigured("No queryset or model defined.")
		if self.distinct:
			self.queryset = queryset.distinct()
		else:
			self.queryset = queryset
		return self.queryset

	def get_model(self):
		if hasattr(self, "model") and self.model is not None:
			model = self.model
		elif hasattr(self, "queryset") and self.queryset is not None:
			model = self.queryset.model
			self.model = model
		else:
			raise ImproperlyConfigured("No queryset or model defined.")
		return model

	def get_items(self, request):
		items = self.get_queryset(request)
		items = self.filter_items(request, items)
		items = self.sort_items(request, items)
		paginator, page, items = self.paginate_items(request, items)
		return (paginator, page, items)

	def get_filters(self, request):
		_search = request.GET.get("_search")
		_export = request.GET.get("_export")
		_print = request.GET.get("_print")
		filters = None

		if _search == "true" or _export == "true" or _print == "true":
			_filters = request.GET.get("filters")
			try:
				filters = _filters and json.loads(_filters)
			except ValueError:
				return None

			if filters is None:
				field = request.GET.get("searchField")
				op = request.GET.get("searchOper")
				data = request.GET.get("searchString")

				if all([field, op, data]):
					filters = {
						"groupOp": "AND",
						"rules": [{ "op": op, "field": field, "data": data }]
					}
		return filters

	def filter_items(self, request, items):
		# TODO: Add option to use case insensitive filters
		# TODO: Add more support for RelatedFields (searching and displaying)
		# FIXME: Validate data types are correct for field being searched.
		filter_map = {
			# jqgrid op: (django_lookup, use_exclude)
			"ne": ("%(field)s__exact", True),
			"bn": ("%(field)s__startswith", True),
			"en": ("%(field)s__endswith",  True),
			"nc": ("%(field)s__contains", True),
			"ni": ("%(field)s__in", True),
			"in": ("%(field)s__in", False),
			"eq": ("%(field)s__exact", False),
			"ieq": ("%(field)s__iexact", False),
			"bw": ("%(field)s__startswith", False),
			"gt": ("%(field)s__gt", False),
			"ge": ("%(field)s__gte", False),
			"lt": ("%(field)s__lt", False),
			"le": ("%(field)s__lte", False),
			"ew": ("%(field)s__endswith", False),
			"cn": ("%(field)s__contains", False),
			"icn": ("%(field)s__icontains", False)
		}
		_filters = self.get_filters(request)
		if _filters is None:
			return items

		q_filters = []
		for rule in _filters["rules"]:
			op, field, data = rule["op"], rule["field"], rule["data"]
			opts = self.get_model()._meta
			field_class = self.lookup_foreign_key_field(opts, field)[0]
			filter_fmt, exclude = filter_map[op]
			filter_str = smart_str(filter_fmt % {"field": field})
			if filter_fmt.endswith("__in"):
				d_split = data.split(",")
				filter_kwargs = {filter_str: data.split(",")}
			else:
				if field_class.get_internal_type() == "BooleanField":
					# Boolean fields have a special treatment
					filter_value = False
					if smart_str(data) == "true": filter_value = True
					filter_kwargs = {filter_str: filter_value}
				else:
					filter_kwargs = {filter_str: smart_str(data)}
			# Append the filters to the list
			if exclude:
				q_filters.append(~models.Q(**filter_kwargs))
			else:
				q_filters.append(models.Q(**filter_kwargs))
		# Filter the data
		if _filters["groupOp"].upper() == "OR":
			filters = reduce(operator.ior, q_filters)
		else:
			filters = reduce(operator.iand, q_filters)
		return items.filter(filters)

	def sort_items(self, request, items):
		sidx = request.GET.get("sidx")
		if sidx is not None:
			sord = request.GET.get("sord")
			order_by = "%s%s" % (sord == "desc" and "-" or "", sidx)
			try:
				items = items.order_by(order_by)
			except FieldError:
				pass
		return items

	def get_paginate_by(self, request):
		rows = request.GET.get("rows", 10)
		try:
			paginate_by = int(rows)
		except ValueError:
			paginate_by = 10
		return paginate_by

	def paginate_items(self, request, items):
		paginate_by = self.get_paginate_by(request)
		if not paginate_by:
			return (None, None, items)

		paginator = Paginator(items, paginate_by,
			allow_empty_first_page=self.allow_empty)
		page = request.GET.get("page", 1)

		try:
			page_number = int(page)
			page = paginator.page(page_number)
		except (ValueError, InvalidPage):
			page = paginator.page(1)
		return (paginator, page, page.object_list)

	def get_data(self, request, as_json=True):
		# Honor the start_empty setting
		_search = request.GET.get("_search", "false")
		if self.start_empty:
			result = {
				"page": 1,
				"total": 1,
				"rows": [],
				"records": 0
			}
		else:
			# Otherwise return the items
			paginator, page, items = self.get_items(request)
			result = {
				"page": page.number,
				"total": paginator.num_pages,
				"rows": items,
				"records": paginator.count
			}
		if as_json:
			return json_encode(result)
		return result

	def get_results(self, request):
		items = self.get_full_queryset(request)
		items = self.filter_items(request, items)
		items = self.sort_items(request, items)
		return items

	def get_json(self, request):
		# Honor the start_empty setting
		if self.start_empty and request.GET.get("_search", "false") == "false":
			return json_encode({
				"page": 1,
				"total": 1,
				"rows": [],
				"records": 0
			})
		# Otherwise return the items
		paginator, page, items = self.get_items(request)
		return json_encode({
			"page": page.number,
			"total": paginator.num_pages,
			"rows": items,
			"records": paginator.count
		})

	def get_html(self, request, template, context):
		from django.template.response import TemplateResponse
		items = self.get_results(request)
		logger.debug(items)
		context.update({"caption": self.caption, "items": items})
		return TemplateResponse(request, template, context)

	def get_default_config(self):
		config = {
			"datatype": "json",
			"autowidth": True,
			"forcefit": True,
			"shrinkToFit": True,
			"jsonReader": {"repeatitems": False},
			"rowNum": 10,
			"rowList": [10, 25, 50, 100],
			"sortname": "id",
			"viewrecords": True,
			"sortorder": "asc",
			"pager": self.pager_id,
			"altRows": True,
			"gridview": True,
			"height": "auto",
			"viewsortcols": True,
			#"multikey": "ctrlKey",
			#"multiboxonly": True,
			#"multiselect": False,
			#"toolbar": [False, "bottom"],
			#"userData": None,
			#"rownumbers": False,
		}
		return config

	def get_url(self):
		return self.url

	def get_caption(self):
		if self.caption is None:
			opts = self.get_model()._meta
			self.caption = opts.verbose_name_plural.capitalize()
		return self.caption

	def get_config(self, as_json=True):
		config = self.get_default_config()
		config.update({
			"url": self.get_url(),
			"caption": self.get_caption(),
			"colModel": self.get_colmodels(),
		})
		if as_json:
			config = json_encode(config)
		return config
	
	def lookup_foreign_key_field(self, options, field_name):
		"""Make a field lookup converting __ into real models fields"""
		if "__" in field_name:
			fk_name, field_name = field_name.split("__", 1)
			fields = [f for f in options.fields if f.name == fk_name]
			if len(fields) > 0:
				field_class = fields[0]
			else:
				raise FieldError("No field %s in %s" % (fk_name, options))
			foreign_model_options = field_class.rel.to._meta
			return self.lookup_foreign_key_field(foreign_model_options, field_name)
		else:
			return options.get_field_by_name(field_name)

	def get_colmodels(self):
		colmodels = []
		opts = self.get_model()._meta
		for field_name in self.get_field_names():
			(field, model, direct, m2m) = self.lookup_foreign_key_field(opts, field_name)
			colmodel = self.field_to_colmodel(field, field_name)
			override = self.colmodel_overrides.get(field_name)

			if override:
				colmodel.update(override)
			colmodels.append(colmodel)
		return colmodels

	def get_field_names(self):
		fields = self.fields
		if not fields:
			fields = [f.name for f in self.get_model()._meta.local_fields]
		return fields

	def field_to_colmodel(self, field, field_name):
		colmodel = {
			"name": field_name,
			"index": field.name,
			"label": field.verbose_name,
			"editable": False
		}
		return colmodel



#
# Django Gems.
#
# Copyright (C) 2011 Pier Luigi Fiorini
# Copyright (C) 2008-2010 Tobias Llipstein
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Adapted from
#		http://code.google.com/p/dojango/source/browse/trunk/dojango/util/__init__.py
#

import os, datetime
from decimal import Decimal

from django.core.serializers.json import DateTimeAwareJSONEncoder
from django.db.models import Model
from django.db.models import ImageField, FileField
from django.db.models.query import QuerySet
from django.http import HttpResponse
from django.utils import simplejson as json
from django.utils.functional import Promise
from django.utils.encoding import force_unicode

def json_encode(data):
	"""
	The main issues with Django's default JSON serializer is that properties that
	had been added to an object dynamically are being ignored (and it also has 
	problems with some models).
	"""
	def _any(data):
		ret = None
		# Ops, we used to check if it is of type list, but that fails 
		# i.e. in the case of django.newforms.utils.ErrorList, which extends
		# the type "list". Oh man, that was a dumb mistake!
		if isinstance(data, list):
			ret = _list(data)
		# Same as for lists above.
		elif isinstance(data, dict):
			ret = _dict(data)
		elif isinstance(data, Decimal):
			# json.dumps() cant handle Decimal
			ret = str(data)
		elif isinstance(data, QuerySet):
			# Actually its the same as a list ...
			ret = _list(data)
		elif isinstance(data, Model):
			ret = _model(data)
		# here we need to encode the string as unicode (otherwise we get utf-16 in the json-response)
		elif isinstance(data, basestring):
			ret = unicode(data)
		# see http://code.djangoproject.com/ticket/5868
		elif isinstance(data, Promise):
			ret = force_unicode(data)
		elif isinstance(data, datetime.datetime):
			# For dojo.date.stamp we convert the dates to use 'T' as separator instead of space
			# i.e. 2008-01-01T10:10:10 instead of 2008-01-01 10:10:10
			ret = str(data).replace(' ', 'T')
		elif isinstance(data, datetime.date):
			ret = str(data)
		elif isinstance(data, datetime.time):
			ret = 'T' + str(data)
		else:
			# Always fallback to a string!
			ret = data
		return ret
	
	def _model(data):
		ret = {}
		# If we only have a model, we only want to encode the fields.
		for f in data._meta.fields:
			# special FileField handling (they can't be json serialized)
			if isinstance(f, ImageField) or isinstance(f, FileField):
				ret[f.attname] = unicode(getattr(data, f.attname))
			else:
				ret[f.attname] = _any(getattr(data, f.attname))
		# And additionally encode arbitrary properties that had been added.
		fields = dir(data.__class__) + ret.keys()
		# ignoring _state and delete properties
		add_ons = [k for k in dir(data) if k not in fields and k not in ('delete', '_state',)]
		for k in add_ons:
			ret[k] = _any(getattr(data, k))
		return ret

	def _list(data):
		ret = []
		for v in data:
			ret.append(_any(v))
		return ret
	
	def _dict(data):
		ret = {}
		for k,v in data.items():
			ret[k] = _any(v)
		return ret
	
	ret = _any(data)
	return json.dumps(ret, cls=DateTimeAwareJSONEncoder)

def json_decode(json_string):
	"""
	This function is just for convenience/completeness (because we have json_encode).
	Sometimes you want to convert a json-string to a python object.
	It throws a ValueError, if the JSON String is invalid.
	"""
	return json.loads(json_string)

def json_view(func):
	"""
	Decorator that serializes data into a JSON string.
	"""
	def _wrap(request, *args, **kwargs):
		response = None
		try:
			data = func(request, *args, **kwargs)
		except KeyboardInterrupt:
			# Allow keyboard interrupts through for debugging.
			raise
		import datetime
		encoded = json_encode(data)
		response = HttpResponse(encoded, mimetype="application/json")
		response["Pragma"] = "no-cache"
		response["Cache-Control"] = "must-revalidate"
		response["If-Modified-Since"] = str(datetime.datetime.now())
		return response
	return _wrap

class JSONResponseMixin(object):
	def render_to_response(self, context):
		"Returns a JSON response containing 'context' as payload."
		return self.get_json_response(self.convert_context_to_json(context))
	
	def get_json_response(self, content, **httpresponse_kwargs):
		"Construct a `HttpResponse` object."
		import datetime
		response = HttpResponse(content,
				mimetype="application/json",
				**httpresponse_kwargs)
		response["Pragma"] = "no-cache"
		response["Cache-Control"] = "must-revalidate"
		response["If-Modified-Since"] = str(datetime.datetime.now())
		return response
	
	def convert_context_to_json(self, context):
		"Convert the context dictionary into a JSON object."
		return json_encode(context)



#
# Django Gems.
#
# Copyright (C) 2011 Pier Luigi Fiorini
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

#
# The urlquote() function was taken from this page:
#    http://djangosnippets.org/snippets/1159/
#
def urlquote(link=None, get={}):
	"""
	This method does both: urlquote() and urlencode()

	urlqoute(): Quote special characters in 'link'

	urlencode(): Map dictionary to query string key=value&...

	HTML escaping is not done.

	Example:

	  urlquote('/wiki/Python_(programming_language)')     --> '/wiki/Python_%28programming_language%29'
	  urlquote('/mypath/', {'key': 'value'})              --> '/mypath/?key=value'
	  urlquote('/mypath/', {'key': ['value1', 'value2']}) --> '/mypath/?key=value1&key=value2'
	  urlquote({'key': ['value1', 'value2']})             --> 'key=value1&key=value2'
	"""
	from django.utils.http import urlquote  as django_urlquote
	from django.utils.http import urlencode as django_urlencode
	from django.utils.datastructures import MultiValueDict

	assert link or get
	if isinstance(link, dict):
		# urlqoute({'key': 'value', 'key2': 'value2'}) --> key=value&key2=value2
		assert not get, get
		get=link
		link=''
	assert isinstance(get, dict), 'wrong type "%s", dict required' % type(get)
	assert not (link.startswith('http://') or link.startswith('https://')), \
		'This method should only quote the url path. It should not start with http(s)://  (%s)' % (
		link)
	if get:
		# http://code.djangoproject.com/ticket/9089
		if isinstance(get, MultiValueDict):
			get=get.lists()
		if link:
			link='%s?' % django_urlquote(link)
		return u'%s%s' % (link, django_urlencode(get, doseq=True))
	else:
		return django_urlquote(link)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name="django-gems",
	version="0.3.0",
	description="Django utilities library",
	long_description="""Contents of this library:
		* jqgrid integration with Django
		* JSONResponseMixIn
		* JSON encoding and decoding functions
		* JSON view decorator
		* Excel view decorator
		* Raw template tag that doesn't parse its content, useful for jQuery Template code which uses the same notation of Django templates
		* Currency formatter for templates
		* Multiple querysets
		* Safe string formatting for templates (useful for translations)""",
	author="Pier Luigi Fiorini",
	author_email="pierluigi.fiorini@gmail.com",
	maintainer="Pier Luigi Fiorini",
	maintainer_email="pierluigi.fiorini@gmail.com",
	url="http://plfiorini.github.com/django-gems",
	download_url="http://github.com/plfiorini/django-gems/tarball/master",
	packages=find_packages(),
	license="BSD",
	classifiers=["Development Status :: 5 - Production/Stable",
		"Intended Audience :: Developers",
		"Framework :: Django",
		"License :: OSI Approved :: BSD License",
		"Operating System :: OS Independent",
		"Topic :: Internet"],
	)



#
# Django Gems.
#
# Copyright (C) 2012 Pier Luigi Fiorini
#
# Author(s):
#   Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#   t_rybik
#
# From: http://djangosnippets.org/snippets/1933/
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from itertools import chain, dropwhile
from operator import mul, attrgetter, __not__

from django.db.models.query import REPR_OUTPUT_SIZE, EmptyQuerySet

def mul_it(it1, it2):
	"""
	Element-wise iterables multiplications.
	"""
	assert len(it1) == len(it2),\
		   "Can not element-wise multiply iterables of different length."
	return map(mul, it1, it2)

def chain_sing(*iterables_or_items):
	"""
	As itertools.chain except that if an argument is not iterable then chain it
	as a singleton.
	"""
	for iter_or_item in iterables_or_items:
		if hasattr(iter_or_item, "__iter__"):
			for item in iter_or_item:
				yield item
		else:
			yield iter_or_item

class IableSequence(object):
	"""
	Wrapper for sequence of iterable and indexable by non-negative integers
	objects. That is a sequence of objects which implement __iter__, __len__ and
	__getitem__ for slices, ints and longs.

	Note: not a Django-specific class.
	"""
	def __init__(self, *args, **kwargs):
		self.iables = args # wrapped sequence
		self._len = None # length cache
		self._collapsed = [] # collapsed elements cache

	def __len__(self):
		if not self._len:
			self._len = sum(len(iable) for iable in self.iables)
		return self._len


	def __iter__(self):
		return chain(*self.iables)

	def __nonzero__(self):
		try:
			iter(self).next()
		except StopIteration:
			return False
		return True

	def _collect(self, start=0, stop=None, step=1):
		if not stop:
			stop = len(self)
		sub_iables = []
		# collect sub sets
		it = self.iables.__iter__()
		try:
			while stop > start:
				i = it.next()
				i_len = len(i)
				if i_len > start:
					# no problem with 'stop' being too big
					sub_iables.append(i[start:stop:step])
				start = max(0, start-i_len)
				stop -= i_len
		except StopIteration:
			pass
		return sub_iables

	def __getitem__(self, key):
		"""
		Preserves wrapped indexable sequences.
		Does not support negative indices.
		"""
		# params validation
		if not isinstance(key, (slice, int, long)):
			raise TypeError
		assert ((not isinstance(key, slice) and (key >= 0))
				or (isinstance(key, slice) and (key.start is None or key.start >= 0)
					and (key.stop is None or key.stop >= 0))), \
				"Negative indexing is not supported."
		# initialization
		if isinstance(key, slice):
			start, stop, step = key.indices(len(self))
			ret_item=False
		else: # isinstance(key, (int,long))
			start, stop, step = key, key+1, 1
			ret_item=True
		# collect sub sets
		ret_iables = self._collect(start, stop, step)
		# return the simplest possible answer
		if not len(ret_iables):
			if ret_item:
				raise IndexError("'%s' index out of range" % self.__class__.__name__)
			return ()
		if ret_item:
			# we have exactly one query set with exactly one item
			assert len(ret_iables) == 1 and len(ret_iables[0]) == 1
			return ret_iables[0][0]
		# otherwise we have more then one item in at least one query set
		if len(ret_iables) == 1:
			return ret_iables[0]

		# Note: this can't be self.__class__ instead of IableSequence; exemplary
		# cause is that indexing over query sets returns lists so we can not
		# return QuerySetSequence by default. Some type checking enhancement can
		# be implemented in subclasses.
		return IableSequence(*ret_iables)

	def collapse(self, stop=None):
		"""
		Collapses sequence into a list.
		Try to do it effectively with caching.
		"""
		if not stop:
			stop = len(self)

		# if we already calculated sufficient collapse then return it
		if len(self._collapsed) >= stop:
			return self._collapsed[:stop]

		# otherwise collapse only the missing part
		items = self._collapsed
		sub_iables = self._collect(len(self._collapsed), stop)
		for sub_iable in sub_iables:
			items+=sub_iable

		# cache new collapsed items
		self._collapsed = items
		return self._collapsed

	def __repr__(self):
		# get +1 element for the truncation msg if applicable
		items = self.collapse(stop=REPR_OUTPUT_SIZE+1)
		if len(items) > REPR_OUTPUT_SIZE:
			items[-1] = "...(remaining elements truncated)..."
		return repr(items)

class QuerySetSequence(IableSequence):
	"""
	Wrapper for the query sets sequence without the restriction on the identity
	of the base models.
	"""
	def count(self):
		if not self._len:
			self._len = sum(qs.count() for qs in self.iables)
		return self._len

	def __len__(self):
		# override: use DB effective count's instead of len()
		return self.count()

	def order_by(self, *field_names):
		"""
		Returns a list of the QuerySetSequence items with the ordering changed.
		"""
		# construct a comparator function based on the field names prefixes
		reverses = [1] * len(field_names)
		field_names = list(field_names)
		for i in range(len(field_names)):
			field_name = field_names[i]
			if field_name[0] == '-':
				reverses[i] = -1
				field_names[i] = field_name[1:]
		# wanna iterable and attrgetter returns single item if 1 arg supplied
		fields_getter = lambda i: chain_sing(attrgetter(*field_names)(i))

		# comparator gets the first non-zero value of the field comparison
		# results taking into account reverse order for fields prefixed with '-'
		comparator = lambda i1, i2:\
			dropwhile(__not__,
			  mul_it(map(cmp, fields_getter(i1), fields_getter(i2)), reverses)
			).next()

		# return new sorted list
		return sorted(self.collapse(), cmp=comparator)

	def filter(self, *args, **kwargs):
		"""
		Returns a new QuerySetSequence or instance with the args ANDed to the
		existing set.

		QuerySetSequence is simplified thus result actually can be one of:
		QuerySetSequence, QuerySet, EmptyQuerySet.
		"""
		return self._filter_or_exclude(False, *args, **kwargs)

	def exclude(self, *args, **kwargs):
		"""
		Returns a new QuerySetSequence instance with NOT (args) ANDed to the
		existing set.

		QuerySetSequence is simplified thus result actually can be one of:
		QuerySetSequence, QuerySet, EmptyQuerySet.
		"""
		return self._filter_or_exclude(True, *args, **kwargs)

	def _simplify(self, qss=None):
		"""
		Returns QuerySetSequence, QuerySet or EmptyQuerySet depending on the
		contents of items, i.e. at least two non empty QuerySets, exactly one
		non empty QuerySet and all empty QuerySets respectively.

		Does not modify original QuerySetSequence.
		"""
		not_empty_qss = filter(None, qss if qss else self.iables)

		if not len(not_empty_qss):
			return EmptyQuerySet()

		if len(not_empty_qss) == 1:
			return not_empty_qss[0]

		return QuerySetSequence(*not_empty_qss)

	def _filter_or_exclude(self, negate, *args, **kwargs):
		"""
		Maps _filter_or_exclude over QuerySet items and simplifies the result.
		"""
		# each Query set is cloned separately
		return self._simplify(*map(lambda qs:
			qs._filter_or_exclude(negate, *args, **kwargs), self.iables))

	def exists(self):
		for qs in self.iables:
			if qs.exists():
				return True
		return False



#
# Django Gems.
#
# Copyright (C) 2011 Pier Luigi Fiorini
#
# Author: Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#	1. Redistributions of source code must retain the above copyright notice,
#	   this list of conditions and the following disclaimer.
#
#	2. Redistributions in binary form must reproduce the above copyright
#	   notice, this list of conditions and the following disclaimer in the
#	   documentation and/or other materials provided with the distribution.
#
#	3. Neither the name of the project nor the names of its contributors may
#	   be used to endorse or promote products derived from this software
#	   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Taken from
#		http://djangosnippets.org/snippets/1825/
#

import locale

from django.template import Library

register = Library()

@register.filter
def currency(value, arg="", symbol=True):
	"""
	Currency formatting template filter.

	Takes a number -- integer, float, decimal -- and formats it according to
	the locale specified as the template tag argument (arg). Examples:

	  * {{ value|currency }}
	  * {{ value|currency:"en_US" }}
	  * {{ value|currency:"pt_BR" }}
	  * {{ value|currency:"pt_BR.UTF8" }}

	If the argument is omitted, the default system locale will be used.

	The third parameter, symbol, controls whether the currency symbol will be
	printed or not. Defaults to true.

	As advised by the Django documentation, this template won't raise
	exceptions caused by wrong types or invalid locale arguments. It will
	return an empty string instead.

	Be aware that currency formatting is not possible using the 'C' locale.
	This function will fall back to 'en_US.UTF8' in this case.
	"""
	saved = '.'.join([x for x in locale.getlocale() if x]) or (None, None)
	given = arg and ('.' in arg and str(arg) or str(arg) + ".UTF-8")

	# Workaround for Python bug 1699853 and other possibly related bugs.
	if '.' in saved and saved.split('.')[1].lower() in ("utf", "utf8"):
		saved = saved.split('.')[0] + ".UTF-8"

	if saved == (None, None) and given == '':
		given = "en_US.UTF-8"

	try:
		locale.setlocale(locale.LC_ALL, given)
		return locale.currency(value or 0, symbol, True)
	except (TypeError, locale.Error):
		return ""
	finally:
		locale.setlocale(locale.LC_ALL, saved)
